"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.buildProject = void 0;
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const glob_1 = require("glob");
const ejs_1 = __importDefault(require("ejs"));
const templateFile = (fileName, replacements) => {
    let fileContent = node_fs_1.default.readFileSync(fileName, "utf8").toString();
    let outputFileName = fileName;
    // Allow for EJS templates if there is logic required to process the template
    if (fileName.endsWith(".ejs")) {
        node_fs_1.default.unlinkSync(fileName);
        outputFileName = fileName.replace(".ejs", "");
        fileContent = ejs_1.default.render(fileContent, replacements);
    }
    else {
        fileContent = Object.entries(replacements).reduce((acc, [key, value]) => {
            return acc.replace(new RegExp(`({{${key}}}|{{ ${key} }})`, "g"), value?.toString() ?? "");
        }, fileContent);
    }
    node_fs_1.default.writeFileSync(outputFileName, fileContent);
};
// required for npm publish
const renameGitignore = (projectName) => {
    if (node_fs_1.default.existsSync(node_path_1.default.normalize(`${projectName}/gitignore`))) {
        node_fs_1.default.renameSync(node_path_1.default.normalize(`${projectName}/gitignore`), node_path_1.default.normalize(`${projectName}/.gitignore`));
    }
};
const buildProfiler = ({ type, framework, name, css, port, withZephyr, }) => {
    const profiler = {
        NAME: name,
        FRAMEWORK: framework,
        SAFE_NAME: name.replace(/-/g, "_").trim(),
        WITH_ZEPHYR: withZephyr,
    };
    if (type === "API" || type === "Application") {
        profiler.PORT = port;
    }
    if (type === "Application") {
        const isTailwind = css === "Tailwind";
        profiler.CONTAINER = isTailwind
            ? "mt-10 text-3xl mx-auto max-w-6xl"
            : "container";
        profiler.CSS = isTailwind ? "Tailwind" : "Empty CSS";
    }
    return profiler;
};
// I for the life of me, could not get ncp to copy the directory fast enough to
// get the template replacements handled properly.
// So I made this hand rolled function to do it
const copyDirSync = (sourceDir, targetDir) => {
    if (!node_fs_1.default.existsSync(targetDir)) {
        node_fs_1.default.mkdirSync(targetDir, { recursive: true });
    }
    const files = node_fs_1.default.readdirSync(sourceDir);
    files.forEach((file) => {
        const sourcePath = node_path_1.default.join(sourceDir, file);
        const targetPath = node_path_1.default.join(targetDir, file);
        const stats = node_fs_1.default.statSync(sourcePath);
        if (stats.isDirectory()) {
            copyDirSync(sourcePath, targetPath);
        }
        else {
            node_fs_1.default.copyFileSync(sourcePath, targetPath);
        }
    });
};
const buildProject = async (project) => {
    const { name, framework, type, withZephyr } = project;
    const tempDir = type.toLowerCase();
    const profiler = buildProfiler(project);
    if (type === "Application") {
        copyDirSync(node_path_1.default.join(__dirname, `../templates/${tempDir}/${framework}`), name);
        const pkg = node_fs_1.default.readFileSync(node_path_1.default.join(name, "package.json"), "utf8");
        const packageJSON = JSON.parse(pkg);
        packageJSON.devDependencies = packageJSON.devDependencies || {};
        if (project.css === "Tailwind") {
            await copyDirSync(node_path_1.default.join(__dirname, "../templates/application-extras/tailwind"), name);
            packageJSON.devDependencies["@tailwindcss/postcss"] = "^4.0.3";
            packageJSON.devDependencies["tailwindcss"] = "^4.0.3";
        }
        if (withZephyr) {
            packageJSON.dependencies["zephyr-rspack-plugin"] = "^0.0.32";
        }
        await node_fs_1.default.writeFileSync(node_path_1.default.join(name, "package.json"), JSON.stringify(packageJSON, null, 2));
    }
    if (type === "Library") {
        await copyDirSync(node_path_1.default.join(__dirname, `../templates/${tempDir}/typescript`), name);
    }
    if (type === "API") {
        await copyDirSync(node_path_1.default.join(__dirname, `../templates/server/${framework}`), name);
    }
    renameGitignore(name);
    const files = glob_1.glob.sync(`${name}/**/*`);
    for (const file of files) {
        if (node_fs_1.default.lstatSync(file).isFile()) {
            templateFile(file, profiler);
        }
    }
};
exports.buildProject = buildProject;
